@@ -12,6 +12,10 @@
 
 using namespace ut;
 
+static_assert(glz::is_atomic<glz::guard<int>>);
+static_assert(glz::read_supported<glz::guard<int>, glz::JSON>);
+static_assert(glz::read_supported<glz::guard<int>, glz::JSON>);
+
 suite atom_tests = [] {
    "construction"_test = [] {
       glz::guard<int> a;
@@ -220,6 +224,18 @@ suite atom_tests = [] {
       expect(old == 100) << "Exchange with acq_rel memory order should work";
       expect(a.load() == 200) << "Value should be updated";
    };
+
+   "json read/write"_test = [] {
+      glz::guard<int> a(42);
+
+      std::string buffer{};
+      expect(not glz::write_json(a, buffer));
+      expect(buffer == "42");
+
+      a = 100;
+      expect(not glz::read_json(a, buffer));
+      expect(a == 42);
+   };
 };
 
 suite async_string_tests = [] {
@@ -1541,7 +1557,7 @@ suite additional_async_vector_tests = [] {
                   [[maybe_unused]] auto temp_cit = proxy.cbegin() + i;
                }
             }
-            catch (const std::exception& e) {
+            catch (const std::exception&) {
                errors++;
             }
          }
@@ -1564,7 +1580,7 @@ suite additional_async_vector_tests = [] {
                   errors++;
                }
             }
-            catch (const std::exception& e) {
+            catch (const std::exception&) {
                errors++;
             }
          }
@@ -1580,7 +1596,7 @@ suite additional_async_vector_tests = [] {
                *it = counter;
                counter++;
             }
-            catch (const std::exception& e) {
+            catch (const std::exception&) {
                errors++;
             }
          }
@@ -1617,7 +1633,7 @@ suite additional_async_vector_tests = [] {
                   {
                      auto proxy = vec.write();
                      auto insert_pos = proxy.begin();
-                     std::advance(insert_pos, std::min(pos, proxy.size()));
+                     std::advance(insert_pos, (std::min)(pos, proxy.size()));
 
                      // Insert a new value
                      auto it = proxy.insert(insert_pos, t * 1000 + i);
@@ -1640,7 +1656,7 @@ suite additional_async_vector_tests = [] {
                      proxy.erase(erase_iter);
                   }
                }
-               catch (const std::exception& e) {
+               catch (const std::exception&) {
                   errors++;
                }
             }
@@ -1675,7 +1691,7 @@ suite additional_async_vector_tests = [] {
                   {
                      auto proxy = vec.write();
                      auto insert_pos = proxy.begin();
-                     std::advance(insert_pos, std::min(pos, proxy.size()));
+                     std::advance(insert_pos, (std::min)(pos, proxy.size()));
 
                      // Create a string and vector for the complex object
                      std::string data = "Thread " + std::to_string(t) + " Item " + std::to_string(i);
@@ -1696,7 +1712,7 @@ suite additional_async_vector_tests = [] {
                   // Small delay to increase chance of contention
                   std::this_thread::sleep_for(std::chrono::microseconds(std::rand() % 100));
                }
-               catch (const std::exception& e) {
+               catch (const std::exception&) {
                   errors++;
                }
             }
@@ -1757,7 +1773,7 @@ suite additional_async_vector_tests = [] {
                      completed_operations++;
                   }
                }
-               catch (const std::exception& e) {
+               catch (const std::exception&) {
                   // Error, but not necessarily a deadlock
                }
             }
@@ -1802,7 +1818,7 @@ suite additional_async_vector_tests = [] {
                   it->access(); // Access the object to increment counter
                }
             }
-            catch (const std::exception& e) {
+            catch (const std::exception&) {
                errors++;
             }
          }
@@ -1834,7 +1850,7 @@ suite additional_async_vector_tests = [] {
                counter++;
                std::this_thread::sleep_for(std::chrono::microseconds(100));
             }
-            catch (const std::exception& e) {
+            catch (const std::exception&) {
                errors++;
             }
          }
@@ -1883,7 +1899,7 @@ suite additional_async_vector_tests = [] {
                resize_count++;
                std::this_thread::sleep_for(std::chrono::milliseconds(10));
             }
-            catch (const std::exception& e) {
+            catch (const std::exception&) {
                errors++;
             }
          }
@@ -1901,7 +1917,7 @@ suite additional_async_vector_tests = [] {
                      sum += proxy[i];
                   }
                }
-               catch (const std::exception& e) {
+               catch (const std::exception&) {
                   errors++;
                }
             }
@@ -1924,7 +1940,7 @@ suite additional_async_vector_tests = [] {
                   }
                   counter++;
                }
-               catch (const std::exception& e) {
+               catch (const std::exception&) {
                   errors++;
                }
             }
@@ -1990,7 +2006,7 @@ suite additional_async_vector_tests = [] {
                      auto proxy = vec.write();
                      size_t pos = proxy.empty() ? 0 : (gen() % proxy.size());
                      auto it = proxy.begin();
-                     std::advance(it, std::min(pos, proxy.size()));
+                     std::advance(it, (std::min)(pos, proxy.size()));
                      proxy.insert(it, t);
                      break;
                   }
@@ -2047,4 +2063,202 @@ suite additional_async_vector_tests = [] {
    };
 };
 
+struct Point
+{
+   int x;
+   int y;
+
+   bool operator==(const Point& other) const { return x == other.x && y == other.y; }
+
+   struct glaze
+   {
+      using T = Point;
+      static constexpr auto value = glz::object("x", &T::x, "y", &T::y);
+   };
+};
+
+struct User
+{
+   std::string name;
+   int age;
+   std::vector<std::string> hobbies;
+
+   bool operator==(const User& other) const
+   {
+      return name == other.name && age == other.age && hobbies == other.hobbies;
+   }
+
+   struct glaze
+   {
+      using T = User;
+      static constexpr auto value = glz::object("name", &T::name, "age", &T::age, "hobbies", &T::hobbies);
+   };
+};
+
+suite async_vector_json_tests = [] {
+   // JSON serialization/deserialization tests
+   "async_vector write_json / read_json"_test = [] {
+      glz::async_vector<int> v;
+      v.push_back(1);
+      v.push_back(2);
+      v.push_back(3);
+      v.push_back(4);
+      v.push_back(5);
+
+      std::string buffer{};
+
+      // write_json returns a status code: false means success, true means error
+      expect(not glz::write_json(v, buffer)) << "Failed to serialize async_vector";
+      // The JSON for a vector of integers should be an array of numbers
+      expect(buffer == "[1,2,3,4,5]") << buffer;
+
+      glz::async_vector<int> result;
+      expect(not glz::read_json(result, buffer)) << "Failed to deserialize async_vector";
+
+      auto result_reader = result.read();
+      expect(result_reader->size() == 5);
+      expect((*result_reader)[0] == 1);
+      expect((*result_reader)[1] == 2);
+      expect((*result_reader)[2] == 3);
+      expect((*result_reader)[3] == 4);
+      expect((*result_reader)[4] == 5);
+   };
+
+   // Test an empty vector's serialization
+   "async_vector empty serialization"_test = [] {
+      glz::async_vector<int> v;
+      std::string buffer{};
+
+      expect(not glz::write_json(v, buffer));
+      // An empty array in JSON
+      expect(buffer == "[]") << buffer;
+
+      glz::async_vector<int> result;
+      result.push_back(99);
+      result.push_back(100); // Non-empty to start
+      expect(not glz::read_json(result, buffer));
+      expect(result.empty());
+   };
+
+   // Test serialization with custom objects
+   "async_vector custom object serialization"_test = [] {
+      glz::async_vector<Point> points;
+      points.push_back({1, 2});
+      points.push_back({3, 4});
+      points.push_back({5, 6});
+
+      std::string buffer{};
+      expect(not glz::write_json(points, buffer)) << "Failed to serialize custom objects in async_vector";
+
+      glz::async_vector<Point> result;
+      expect(not glz::read_json(result, buffer)) << "Failed to deserialize custom objects in async_vector";
+
+      auto result_reader = result.read();
+      expect(result_reader->size() == 3);
+      expect((*result_reader)[0] == Point{1, 2});
+      expect((*result_reader)[1] == Point{3, 4});
+      expect((*result_reader)[2] == Point{5, 6});
+   };
+
+   // Test serialization with nested async_vector
+   "async_vector nested serialization"_test = [] {
+      glz::async_vector<glz::async_vector<int>> nested;
+
+      // Create and add inner vectors
+      glz::async_vector<int> inner1;
+      inner1.push_back(1);
+      inner1.push_back(2);
+      inner1.push_back(3);
+
+      glz::async_vector<int> inner2;
+      inner2.push_back(4);
+      inner2.push_back(5);
+
+      glz::async_vector<int> inner3;
+      inner3.push_back(6);
+
+      nested.push_back(std::move(inner1));
+      nested.push_back(std::move(inner2));
+      nested.push_back(std::move(inner3));
+
+      std::string buffer{};
+      expect(not glz::write_json(nested, buffer)) << "Failed to serialize nested async_vector";
+
+      // JSON should be a nested array
+      expect(buffer == "[[1,2,3],[4,5],[6]]") << buffer;
+
+      glz::async_vector<glz::async_vector<int>> result;
+      expect(not glz::read_json(result, buffer)) << "Failed to deserialize nested async_vector";
+
+      auto result_reader = result.read();
+      expect(result_reader->size() == 3);
+
+      auto inner1_reader = (*result_reader)[0].read();
+      expect(inner1_reader->size() == 3);
+      expect((*inner1_reader)[0] == 1);
+      expect((*inner1_reader)[1] == 2);
+      expect((*inner1_reader)[2] == 3);
+
+      auto inner2_reader = (*result_reader)[1].read();
+      expect(inner2_reader->size() == 2);
+      expect((*inner2_reader)[0] == 4);
+      expect((*inner2_reader)[1] == 5);
+
+      auto inner3_reader = (*result_reader)[2].read();
+      expect(inner3_reader->size() == 1);
+      expect((*inner3_reader)[0] == 6);
+   };
+
+   // Test with more complex JSON structures
+   "async_vector complex JSON structures"_test = [] {
+      // Create test data
+      glz::async_vector<User> users;
+      users.push_back({"Alice", 30, {"reading", "hiking"}});
+      users.push_back({"Bob", 25, {"gaming", "coding", "music"}});
+
+      std::string buffer{};
+      expect(not glz::write_json(users, buffer)) << "Failed to serialize complex structure";
+
+      // Check JSON format (approximate)
+      expect(buffer.find("Alice") != std::string::npos);
+      expect(buffer.find("Bob") != std::string::npos);
+      expect(buffer.find("reading") != std::string::npos);
+      expect(buffer.find("gaming") != std::string::npos);
+
+      // Deserialize
+      glz::async_vector<User> result;
+      expect(not glz::read_json(result, buffer)) << "Failed to deserialize complex structure";
+
+      auto reader = result.read();
+      expect(reader->size() == 2);
+      expect((*reader)[0] == User{"Alice", 30, {"reading", "hiking"}});
+      expect((*reader)[1] == User{"Bob", 25, {"gaming", "coding", "music"}});
+   };
+
+   // Test JSON serialization with pretty print
+   "async_vector pretty print JSON"_test = [] {
+      glz::async_vector<int> v;
+      v.push_back(1);
+      v.push_back(2);
+      v.push_back(3);
+
+      std::string buffer{};
+      expect(not glz::write<glz::opts{.prettify = true}>(v, buffer)) << "Failed to serialize with pretty print";
+
+      // Should include newlines and indentation
+      expect(buffer.find("\n") != std::string::npos);
+      expect(buffer.find(" ") != std::string::npos);
+
+      // Should still deserialize correctly
+      glz::async_vector<int> result;
+      expect(not glz::read_json(result, buffer)) << "Failed to deserialize pretty-printed JSON";
+
+      auto reader = result.read();
+      expect(reader->size() == 3);
+      expect((*reader)[0] == 1);
+      expect((*reader)[1] == 2);
+      expect((*reader)[2] == 3);
+   };
+};
+
 int main() {}
